namespace Quadratic.Carto.Orbital;

/// <summary>
/// Represents the basic properties of a celestial body.
/// </summary>
/// <param name="Mass">The mass of the body, in kilograms</param>
/// <param name="Radius">The radius of the body, in meters</param>
/// <param name="RotationPeriod">The rotation period of the body, in seconds</param>
public sealed record CelestialBody(
    double Mass,
    double Radius,
    double RotationPeriod
)
{
    /// <summary>
    /// The gravitational parameter of the body, mu = G(M+m), in cubic meters per second squared.
    /// </summary>
    public double Mu => PhysicalConstants.GravitationalConstant * Mass;

    /// <summary>
    /// The surface gravity of the body, in meters per second squared.
    /// </summary>
    public double SurfaceGravity => Mu / (Radius * Radius);

    /// <summary>
    /// Get the celestial body from its surface gravity, radius, and rotation period.
    /// </summary>
    /// <param name="surfaceGravity"></param>
    /// <param name="radius"></param>
    /// <param name="rotationPeriod"></param>
    /// <returns></returns>
    public static CelestialBody FromSurfaceGravity(double surfaceGravity, double radius, double rotationPeriod)
    {
        var mass = surfaceGravity * radius * radius / PhysicalConstants.GravitationalConstant;
        return new CelestialBody(mass, radius, rotationPeriod);
    }

    public static CelestialBody Earth = new CelestialBody(5.972e24, 6371e3, 86164.1);

    public override string ToString()
    {
        return $"CelestialBody(Mass: {Mass}, Radius: {Radius}, RotationPeriod: {RotationPeriod})";
    }
}
