using System;
using Vim.Math3d;

namespace Quadratic.Carto.Orbital;

/// <summary>
/// A list of keplerian orbital elements, representing the orbit around a celestial body.
/// </summary>
/// <remarks>
/// https://orbital-mechanics.space/classical-orbital-elements/orbital-elements-and-the-state-vector.html
/// </remarks>
public sealed class KeplerianElements(
    double ecc, double sma, double inc,
    double lan, double ape, double ta
)
{
    /// <summary>
    /// Eccentricity of the orbit.
    /// </summary>
    public double ecc = ecc;

    /// <summary>
    /// Semi-major axis of the orbit, in meters.
    /// </summary>
    public double sma = sma;

    /// <summary>
    /// Inclination of the orbit, in radians [0, π).
    /// </summary>
    public double inc = inc;

    /// <summary>
    /// Longitude of the ascending node, in radians [0, 2π).
    /// </summary>
    public double lan = lan;

    /// <summary>
    /// Argument of periapsis, in radians [0, 2π).
    /// </summary>
    public double ape = ape;

    /// <summary>
    /// True anomaly, in radians [0, 2π).
    /// </summary>
    public double ta = ta;

    public double IncDegree => inc * Math.PI / 180;
    public double LanDegree => lan * Math.PI / 180;
    public double ApeDegree => ape * Math.PI / 180;
    public double TaDegree => ta * Math.PI / 180;

    public double Apoapsis => sma * (1 + ecc);
    public double Periapsis => sma * (1 - ecc);

    public double Period(double mu)
    {
        return 2 * Math.PI * Math.Sqrt(sma * sma * sma / mu);
    }

    public override string ToString()
    {
        return $"KeplerianElements(ecc: {ecc:F6}, sma: {sma:F6}, inc: {inc:F6}, lan: {lan:F6}, ape: {ape:F6}, ta: {ta:F6})";
    }

    /// <summary>
    /// Returns the position and velocity of the body at the given time.
    /// </summary>
    /// <param name="mu">The standard gravitational parameter, mu = G(M+m)</param>
    /// <param name="epoch">The time of the query</param>
    /// <returns></returns>
    public DStateVector MotionStateAt(double mu, double epoch)
    {
        throw new NotImplementedException();
    }

    public static KeplerianElements FromMotionState(DStateVector state, double mu)
    {
        DVector3 position = state.Position;
        DVector3 velocity = state.Velocity;

        double radius = position.Length();
        double speed = velocity.Length();
        double vRadial = DVector3.Dot(position, velocity) / radius;
        double vTangential = Math.Sqrt(speed * speed - vRadial * vRadial);

        DVector3 angularMomentum = position.Cross(velocity);
        double h = angularMomentum.Length();

        double inclination = Math.Acos(angularMomentum.Z / h);

        DVector3 kHat = new DVector3(0, 0, 1);
        DVector3 n = kHat.Cross(angularMomentum);
        double nNorm = n.Length();
        double lan = Math.Acos(n.X / nNorm);

        DVector3 eVec = velocity.Cross(angularMomentum) / mu - position / radius;
        double ecc = eVec.Length();

        double a = 1 / (2 / radius - speed * speed / mu);

        double ta = Math.Acos((a * (1 - ecc * ecc) / radius - 1) / ecc);

        return new KeplerianElements(ecc, a, inclination, lan, 0, ta);
    }

    public static KeplerianElements EquatorialCircular(double sma, double ta)
    {
        return new KeplerianElements(0, sma, 0, 0, 0, ta);
    }

    public static KeplerianElements Circular(double sma, double inc, double lan, double ta)
    {
        return new KeplerianElements(0, sma, inc, lan, 0, ta);
    }
}

public struct DStateVector(DVector3 position, DVector3 velocity)
{
    public DVector3 Position = position;
    public DVector3 Velocity = velocity;
}
